package org.tekkotsu.ui.editor.editparts;

/**
 * @author asangpet
 */

import java.beans.PropertyChangeEvent;
import java.util.List;

import org.eclipse.draw2d.ChopboxAnchor;
import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.ConnectionAnchor;
import org.eclipse.draw2d.Ellipse;
import org.eclipse.draw2d.EllipseAnchor;
import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.RectangleFigure;
import org.eclipse.draw2d.RoundedRectangle;
import org.eclipse.draw2d.Shape;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.ConnectionEditPart;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.NodeEditPart;
import org.eclipse.gef.Request;
import org.eclipse.swt.graphics.Color;
import org.tekkotsu.ui.editor.editpolicies.MyComponentEditPolicy;
import org.tekkotsu.ui.editor.model.AbstractConnectionModel;
import org.tekkotsu.ui.editor.model.StateNodeModel;
import org.tekkotsu.ui.editor.model.LayoutData;
import org.tekkotsu.ui.editor.resources.ConvexPolygon;
import org.tekkotsu.ui.util.ColorGenerator;
import org.tekkotsu.ui.util.Debugger;

public class StateNodeEditPart 
	extends MyAbstractGraphicalEditPart 
	implements NodeEditPart {
	
	public StateNodeEditPart()
	{
		//nothing
	}
	
	@Override
	protected void createEditPolicies() {
		//Provides delete for nodes
		installEditPolicy(EditPolicy.COMPONENT_ROLE, new MyComponentEditPolicy());
	}
	
	protected IFigure shape, holder;
	protected Label label;
	boolean isSrc = true;

	private ConnectionAnchor getAnchor() {
		StateNodeModel model = (StateNodeModel)getModel();
		if (model.getShape().equals(StateNodeModel.P_SHAPE_STYLE_ELLIPSE)) {
			return new EllipseAnchor(getFigure());
		}
		
		return new ChopboxAnchor(getFigure());
	}
	
	public ConnectionAnchor getSourceConnectionAnchor(ConnectionEditPart connection) {
		return getAnchor();
	}
	
	public ConnectionAnchor getTargetConnectionAnchor(ConnectionEditPart connection) {
		return getAnchor();
	}
	
	public ConnectionAnchor getSourceConnectionAnchor(Request request) {
		return getAnchor();
	}
	
	public ConnectionAnchor getTargetConnectionAnchor(Request request) {
		return getAnchor();
	}
	
	public void propertyChange(PropertyChangeEvent evt) {		
		if (evt.getPropertyName().equals(StateNodeModel.P_ACTIVATE)) {
			try {
				shape.setBackgroundColor(((StateNodeModel)getModel()).getColor());
				setLabelColor(true);
			} catch (Exception e) {
				Debugger.printThrowable(e);
			}
		} else if (evt.getPropertyName().equals(StateNodeModel.P_DEACTIVATE)) {
			try {
				shape.setBackgroundColor(((StateNodeModel)getModel()).getPreviewColor());
				setLabelColor(false);
			} catch (Exception e) {
				Debugger.printThrowable(e);
			}			
		} else if (evt.getPropertyName().equals(StateNodeModel.P_CONSTRAINT)) {
			setDirty();
			refreshVisuals();
		} else if (evt.getPropertyName().equals(StateNodeModel.P_LABEL)) {
			setDirty();
			label.setText((String)evt.getNewValue());
		} else if (evt.getPropertyName().equals(StateNodeModel.P_COLOR)) {
			setDirty();
			shape.setBackgroundColor((Color)evt.getNewValue());
			setLabelColor();
		} else if (evt.getPropertyName().equals(StateNodeModel.P_SOURCE_CONNECTION)) {
			setDirty();
			refreshSourceConnections();
		} else if (evt.getPropertyName().equals(StateNodeModel.P_TARGET_CONNECTION)) {
			setDirty();
			refreshTargetConnections();
		} else if (evt.getPropertyName().equals(StateNodeModel.P_SHAPE)) {
			setDirty();
			resetShape(evt.getNewValue().toString());
		} else if (evt.getPropertyName().equals(LayoutData.P_SOURCE_EXIST)) {
			Debugger.printDebug("SOURCE EXISTS EVENT");
			isSrc = ((Boolean)evt.getNewValue()).booleanValue();
		} else if (evt.getPropertyName().equals("class org.eclipse.swt.graphics.RGB"))	{
			setDirty();
		} else	{
			Debugger.printDebug("UNKOWN PROPERTY CHANGE: " + evt.getPropertyName());
		}
	}
	
	protected IFigure createCustomFigure(IFigure shapeFigure) {
		StateNodeModel model = (StateNodeModel)getModel();
		holder = new Figure();
		
		shape = shapeFigure;
		if (((LayoutData)(getParent().getModel())).isPreview() || (!isSrc)) {
			shape.setBackgroundColor(model.getPreviewColor());
			shape.setBounds(model.getConstraint());			
		} else {
			shape.setBackgroundColor(model.getColor());
			shape.setBounds(model.getConstraint());			
		}
		
		label = new Label();
		label.setText(model.getLabel());
		label.setBounds(model.getConstraint());		
		setLabelColor();
		
		holder.add(shape);
		holder.add(label);
		holder.setBounds(model.getConstraint());		
		return holder;		
	}
	
	private void resetShape(String shapeType) {
		IFigure shapeFigure = null;
		if (shapeType.equals(StateNodeModel.P_SHAPE_STYLE_RECTANGLE)) {
			shapeFigure = new RectangleFigure();
		} else if (shapeType.equals(StateNodeModel.P_SHAPE_STYLE_ELLIPSE)){
			shapeFigure = new Ellipse();
		} else if (shapeType.equals(StateNodeModel.P_SHAPE_STYLE_ROUNDED_RECTANGLE)) {
			shapeFigure = new RoundedRectangle();
		} else if (shapeType.equals(StateNodeModel.P_SHAPE_STYLE_HEXAGON)) {
			shapeFigure = new ConvexPolygon();
		} else {
			//shapeFigure is null
			return;
		}
		shapeFigure.setBackgroundColor(shape.getBackgroundColor());
		shapeFigure.setBounds(shape.getBounds());
				
		if (((LayoutData)(getParent().getModel())).isPreview() || !isSrc) {
			((Shape)shapeFigure).setForegroundColor(ColorConstants.lightGray);			
		} else {
			((Shape)shapeFigure).setForegroundColor(ColorConstants.black);			
		}
		
		holder.remove(shape);
		holder.remove(label);
		shape = shapeFigure;
		holder.add(shape);
		holder.add(label);
	}
	
	@Override
	protected IFigure createFigure() {
		createCustomFigure(new RectangleFigure());
		resetShape(((StateNodeModel)getModel()).getShape());
		return holder;
	}
	
	/**
	 * Adjust label color according to background color of the figure
	 */
	protected void setLabelColor(boolean isActive) {
		if(label == null)
		{
			return;
		}
		
		if (shape == null) {
			label.setForegroundColor(ColorConstants.black);
			return;
		}

		label.setForegroundColor(ColorGenerator.highContrast(shape.getBackgroundColor()));
	}
	
	protected void setLabelColor() {
	    setLabelColor(!(((LayoutData)(((StateNodeModel)getModel())).getParent())).isPreview());
	}
	
	@Override
	protected void refreshVisuals() {
		Rectangle constraint = ((StateNodeModel)getModel()).getConstraint();
		((GraphicalEditPart)getParent()).setLayoutConstraint(this, getFigure(), constraint);
		holder.setBounds(constraint);
		shape.setBounds(constraint);
		label.setBounds(constraint);				
	}
	
	@Override
	protected List<AbstractConnectionModel> getModelSourceConnections() {
		return ((StateNodeModel)getModel()).getModelSourceConnections();
	}
	
	@Override
	protected List<AbstractConnectionModel> getModelTargetConnections() {
		return ((StateNodeModel)getModel()).getModelTargetConnections();
	}
	
	@Override
	public String toString() {
		String result = getClass().getName() + ":" + ((StateNodeModel)getModel()).getId() + 
			":\n\tsrc:" + getSourceConnections() + 
			":\n\tdest:" + getTargetConnections();			
		return result;
	}
}
