package org.tekkotsu.ui.rcp;

import org.eclipse.ui.IFolderLayout;
import org.eclipse.ui.IPageLayout;
import org.eclipse.ui.IPerspectiveFactory;
import org.eclipse.ui.console.*;

import java.io.*;

public class StoryboardPerspective implements IPerspectiveFactory {
	public static final String ID_STORYBOARD_VIEW = "org.tekkotsu.ui.storyboard.views.StoryboardView"; 
	public static final String ID_RUNTIME_VIEW    = "org.tekkotsu.ui.storyboard.views.RuntimeView";
	public static final String ID_IMAGE_VIEW      = "org.tekkotsu.ui.storyboard.views.ImageView";
	public static final String ID_STATE_EDITOR    = "org.tekkotsu.ui.editor.StateMachineEditor";
	
    public StoryboardPerspective() {
    	//nothing
    }

    public void createInitialLayout(IPageLayout layout) {        
		IFolderLayout propRight = layout.createFolder("rightPropFolder", IPageLayout.RIGHT, 0.75f, IPageLayout.ID_EDITOR_AREA);
		
		//Storyboard Timeline
		layout.addView(ID_STORYBOARD_VIEW,IPageLayout.BOTTOM, (float)0.48,IPageLayout.ID_EDITOR_AREA);
			layout.getViewLayout(ID_STORYBOARD_VIEW).setCloseable(false);
		
		//Property Sheet
		propRight.addView(IPageLayout.ID_PROP_SHEET);
			layout.getViewLayout(IPageLayout.ID_PROP_SHEET).setCloseable(false);
		
		//Runtime View
		propRight.addView(ID_RUNTIME_VIEW);
			layout.getViewLayout(ID_RUNTIME_VIEW).setCloseable(false);
		
		//Image View
		layout.addView(ID_IMAGE_VIEW, IPageLayout.BOTTOM, (float)0.48, ID_RUNTIME_VIEW);
			layout.getViewLayout(ID_IMAGE_VIEW).setCloseable(false);
    	
    	initializeConsole();
    }
        
    protected void initializeConsole() {
    	ConsolePlugin console = ConsolePlugin.getDefault();
    	MessageConsole outputConsole = new MessageConsole("Debug Console",null);
    	
    	MessageConsoleStream outs = outputConsole.newMessageStream();
    	console.getConsoleManager().addConsoles(
    			new IConsole[] { outputConsole });
    	
    	StreamRedirect redir = new StreamRedirect(outs);
  
    	System.setOut(redir);   	
    }
    
    static class StreamRedirect extends PrintStream {
    	private MessageConsoleStream console;
    	
    	public StreamRedirect(MessageConsoleStream output) {
    		super(System.err,true);
    		this.console = output;
    	}
    	
    	@Override
		public void println(String s) {
    		super.println(s);
    		console.println(s);
    	}

    }

}

