package org.tekkotsu.ui.rcp;

import org.eclipse.core.runtime.IPath;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.ViewerSorter;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.IPartListener;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IViewReference;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;
import org.eclipse.ui.views.properties.PropertySheet;
import org.tekkotsu.ui.editor.StateMachineEditor;
import org.tekkotsu.ui.editor.model.LayoutData;
import org.tekkotsu.ui.rcp.actions.SaveTraceAction;
import org.tekkotsu.ui.storyboard.actions.LoadTraceAction;
import org.tekkotsu.ui.storyboard.views.ContentProvider;
import org.tekkotsu.ui.storyboard.views.ImageView;
import org.tekkotsu.ui.storyboard.views.ImageViewer;
import org.tekkotsu.ui.storyboard.views.RuntimeView;
import org.tekkotsu.ui.storyboard.views.StoryboardViewer;
import org.tekkotsu.ui.util.Debugger;

public class StoryboardView extends ViewPart implements IPartListener {

	public static final double timeUnit = 1000.0;

	public static StoryboardView view;

	public StoryboardViewer viewer;

	public StateMachineEditor editor;
	public RuntimeView runtime;
	public ContentProvider content;
	public IViewPart propSheet;

	public ImageViewer imageViewer;

	static class ViewLabelProvider extends LabelProvider implements
			ITableLabelProvider {
		public String getColumnText(Object obj, int index) {
			return getText(obj);
		}

		public Image getColumnImage(Object obj, int index) {
			return getImage(obj);
		}

		@Override
		public Image getImage(Object obj) {
			return PlatformUI.getWorkbench().getSharedImages().getImage(
					ISharedImages.IMG_OBJ_ELEMENT);
		}
	}

	static class NameSorter extends ViewerSorter {
		// nothing
	}

	/**
	 * The constructor.
	 */
	public StoryboardView() {
		view = this;
	}

	/**
	 * This is a callback that will allow us to create the viewer and initialize
	 * it.
	 */
	@Override
	public void createPartControl(Composite parent) {
		content = new ContentProvider(this);
		viewer = new StoryboardViewer(parent, this, editor, content,
				imageViewer);
		this.getViewSite().getWorkbenchWindow().getPartService()
				.addPartListener(this);
	}

	public Action makeSaveTraceAction() {
		return new SaveTraceAction(this);
	}

	public Action makeLoadTraceAction() {
		return new LoadTraceAction();
	}

	/**
	 * Passing the focus request to the viewer's control.
	 */
	@Override
	public void setFocus() {
		viewer.getControl().setFocus();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.ui.IPartListener#partActivated(org.eclipse.ui.IWorkbenchPart)
	 */
	public void partActivated(IWorkbenchPart part) {
		if (part == null)
			return;

		IWorkbenchPage page = part.getSite().getPage();

		if (part == this) {
			if (runtime == null) {
				for (IViewReference vref : page.getViewReferences()) {
					IWorkbenchPart p = vref.getPart(true);
					if (p instanceof RuntimeView)
						runtime = (RuntimeView) p;
				}
			}

			page.bringToTop(runtime);
			if( editor != null ) {
				editor.layoutData.deactivate();
			}
		} else if (part == editor) {
			editor.layoutData.activate();
			if (propSheet == null) {
				for (IViewReference vref : page.getViewReferences()) {
					IWorkbenchPart p = vref.getPart(true);
					if (p instanceof PropertySheet)
						propSheet = (PropertySheet) p;
				}
			}

			page.bringToTop(propSheet);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.ui.IPartListener#partBroughtToTop(org.eclipse.ui.IWorkbenchPart
	 * )
	 */
	public void partBroughtToTop(IWorkbenchPart part) {
		// nothing
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.ui.IPartListener#partClosed(org.eclipse.ui.IWorkbenchPart)
	 */
	public void partClosed(IWorkbenchPart part) {
		if (part == editor) {
			editor.props.removePropertyChangeListener(viewer);
			editor.getRobotConnector().props.removePropertyChangeListener(viewer);
			editor.getRobotConnector().props.removePropertyChangeListener(editor);
			editor = null;
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.ui.IPartListener#partDeactivated(org.eclipse.ui.IWorkbenchPart
	 * )
	 */
	public void partDeactivated(IWorkbenchPart part) {
		//nothing
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.ui.IPartListener#partOpened(org.eclipse.ui.IWorkbenchPart)
	 */
	public void partOpened(IWorkbenchPart part) {
		if (part instanceof StateMachineEditor) {
			editor = (StateMachineEditor) part;

			editor.getRobotConnector().props.addPropertyChangeListener(viewer);
			editor.getRobotConnector().props.addPropertyChangeListener(editor);

			editor.props.addPropertyChangeListener(viewer);

			LayoutData layoutData = editor.getLayoutData();
			IPath      path       = layoutData.getPath();
			try {
				content.parseNewTrace(path);
				viewer.setContent(content);
				viewer.setInput(layoutData);
			} catch (Exception e) {
				Debugger.printThrowable(e);
			}
		} else if (part instanceof RuntimeView) {
			runtime = (RuntimeView) part;
		} else if (part instanceof ImageView) {
			viewer.setImageViewer(((ImageView) part).viewer);
		}
	}

}