/*
 * Created on Mar 29, 2005
 */
package org.tekkotsu.ui.storyboard.panels;

import static org.tekkotsu.ui.storyboard.panels.TimeLine.COLOR_BACKGROUND;
import static org.tekkotsu.ui.storyboard.panels.TimeLine.COLOR_BORDER;
import static org.tekkotsu.ui.storyboard.panels.TimeLine.COLOR_CURSOR;

import java.beans.PropertyChangeListener;
import java.util.Collection;
import java.util.NavigableMap;
import java.util.Set;
import java.util.TreeMap;
import java.util.TreeSet;

import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.Panel;
import org.eclipse.draw2d.geometry.Rectangle;
import org.tekkotsu.ui.rcp.StoryboardView;
import org.tekkotsu.ui.storyboard.objects.AbstractViewObject;
import org.tekkotsu.ui.storyboard.objects.LogViewObject;
import org.tekkotsu.ui.storyboard.views.RuntimeView;
import org.tekkotsu.ui.storyboard.views.StoryboardViewer;
import org.tekkotsu.ui.util.Debugger;

/**
 * 
 * @author asangpet
 */
public class LogBar extends AbstractTimePanel implements PropertyChangeListener {
	public static final int HEIGHT = 28;
	public static final int X_POS = 0;
	public static final int Y_POS = TimeLine.HEIGHT;
	public static final int TIME_BUFFER = 250;

	@Override
	protected int getPreferredX() {
		return X_POS;
	}
	@Override
	protected int getPreferredY() {
		return Y_POS;
	}
	@Override
	protected int getPreferredH() {
		return HEIGHT;
	}
	
	/** List of currently selected objects */
	private Set<AbstractViewObject> selectedObjects = new TreeSet<AbstractViewObject>(
			new AbstractViewObject.BeginComparator());
	
	/** List of all children sorted by their start time */
	private NavigableMap<AbstractViewObject,AbstractViewObject> allObjects = new TreeMap<AbstractViewObject,AbstractViewObject>(
			new AbstractViewObject.BeginComparator());
	
	public LogBar(Panel parent, StoryboardViewer viewer, TimeMarker marker) {
		super(parent,viewer,marker);	
	}

	@Override
	public void timeCursorUpdated() {
		final int CUR_TIME = getTimeCursor();
		RuntimeView.TreeParent infoTree = RuntimeView.getInstance().getContent(CUR_TIME);

		// Save old selection
		Set<AbstractViewObject> oldSelection = new TreeSet<AbstractViewObject>(new AbstractViewObject.BeginComparator());
		oldSelection.addAll(selectedObjects);
		
		// Find selected objects
		selectedObjects.clear();
		selectedObjects.addAll(findNear(getOffsetOfTime(CUR_TIME)));

		// select objects that weren't selected last update
		for (AbstractViewObject aobj : selectedObjects) {
			if( !oldSelection.contains(aobj) ) {
				aobj.setActive(true);
			}
			infoTree.addChild(aobj.getRuntimeViewData());
		}
		// deselect objects that were selected last update
		for (AbstractViewObject aobj : oldSelection) {
			if( !selectedObjects.contains(aobj) ) {
				aobj.setActive(false);
			}
		}
		StoryboardView.view.runtime.setContent(infoTree);

		super.timeCursorUpdated();
	}
	
	@Override
	protected void paintFigure(Graphics gc) {
		Debugger.printDebug("[LogBar] paintFigure(" + gc + ")");
		super.paintFigure(gc);

		final Rectangle bounds = getBounds();
		final Rectangle viewbox = viewer.getVisibleViewport();

		final int BX = bounds.x;
		final int BY = bounds.y;
//		final int BW = bounds.width;
		final int BH = bounds.height;
		final int VIEW_L = viewbox.x - 100;
		final int VIEW_R = VIEW_L + viewbox.width + 100;
//		final int CHAR_WIDTH = gc.getFontMetrics().getAverageCharWidth();

		// the background
		gc.setBackgroundColor(colorReg.get(COLOR_BACKGROUND));
		gc.fillRectangle(getBounds());

//		/*
//		 * Draw Event Ticks
//		 */
//		TimeMarker marker = getMarker();
//
//		NumberFormat nf = new DecimalFormat("0.###");
//		nf.setMinimumFractionDigits(2);
//		nf.setMaximumFractionDigits(2);
//		nf.setMaximumIntegerDigits(2);
//		
//		final int START_INDEX = marker.getIndexOfOffsetBefore(VIEW_L);
//		final int END_INDEX   = marker.getIndexOfOffsetAfter(VIEW_R);
//		
//		int clearAfter = 0;
//		for (int i = START_INDEX; i < END_INDEX; i++) {			
//			int time = marker.timeVec.get(i);
//			int x = marker.offsetVec.get(i);
//			
//			if( x + BW < VIEW_L )
//			{
//				Debugger.printDebug("LOGBAR TOO LEFT");
//			}
//			if( x > VIEW_R )
//			{
//				Debugger.printDebug("LOGBAR TOO RIGHT");
//				break;
//			}
//
//			// draw marker
//			gc.setForegroundColor(colorReg.get(COLOR_EVENT));
//			gc.drawLine(BX + x, BY, BX + x, BY + 4);
//			gc.drawLine(BX + x, BY + BH, BX + x, BY + BH - 5);
//
//			// draw timer (if possible)
//			String label = nf.format(time * 0.001f);
//			final int LABEL_W = CHAR_WIDTH * label.length();
//			final int LABEL_X = x - LABEL_W/2 - 1;
//			
//			if ((LABEL_X - clearAfter > 10) ) {
//				gc.setForegroundColor(colorReg.get(COLOR_TEXT));
//				gc.drawString(label, BX + LABEL_X, BY + 6);
//			}
//			clearAfter = LABEL_X + LABEL_W;
//		}
		
		// the border
		gc.setForegroundColor(colorReg.get(COLOR_BORDER));
		gc.drawLine(VIEW_L, BY, VIEW_R, BY);
		gc.drawLine(VIEW_L, BY + BH - 1, VIEW_R, BY + BH - 1);
		
		// draw cursor
		gc.setXORMode(true);
		gc.setBackgroundColor(colorReg.get(COLOR_CURSOR));
		gc.fillRectangle(
				BX + getOffsetOfTime(getTimeCursor()) - AbstractTimePanel.halfgap, BY + 1,
				AbstractTimePanel.gap, BH - 2);
		gc.setXORMode(false);
	}
	
	@Override
	public void update(AbstractViewObject newchild) {
		LogViewObject child = (LogViewObject)newchild;
		
		final AbstractViewObject OLD = allObjects.get(child);
		if( OLD != null ) {
			allObjects.remove(OLD);
			remove(OLD);
		}
		
		add(child);
		allObjects.put(child,child);

		// scan for close figure
		int child_x = getOffsetOfTime(child.getModel().getTime());
		int idx = 0;
		for (AbstractViewObject clog : findNear(child_x)) {
			if( clog == child ) {
				continue;
			}
			
			int clog_x = getOffsetOfTime(clog.getModel().getStart());
			if (Math.abs(clog_x - child_x) < 8) {
				clog.setLayoutIndex(++idx);
			}
		}
		child.setLayoutIndex(++idx);
	}

	private Collection<AbstractViewObject> findNear(int x) {		
		AbstractViewObject low = AbstractViewObject.getComparatorQuery(getTimeOfOffset(x)-TIME_BUFFER);
		AbstractViewObject high = AbstractViewObject.getComparatorQuery(getTimeOfOffset(x)+TIME_BUFFER);
		
		Collection<AbstractViewObject> near = allObjects.tailMap(low).headMap(high).values();
		
		return near;
	}

	@Override
	public void reset() {
		super.reset();
		super.removeAll();
		selectedObjects.clear();
		allObjects.clear();
	}
}
