package org.tekkotsu.ui.storyboard.views;

import java.text.MessageFormat;

import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.part.ViewPart;
import org.tekkotsu.ui.storyboard.model.ImageModel;
import org.tekkotsu.ui.util.FilesystemUtil;

public class ImageView extends ViewPart {
    public ImageViewer viewer;

    Action saveAction;

    static class ViewContentProvider implements IStructuredContentProvider {
        ImageModel model = null;

        public Object[] getElements(Object inputElement) {
            return null;
        }

        public void inputChanged(Viewer v, Object oldInput, Object newInput) {
        	//nothing
        }

        public void dispose() {
        	//nothing
        }

        public void setContent(ImageModel img) {
            model = img;
        }

        public ImageModel getDefaultModel() {
            return model;
        }
    }

    /**
     * The constructor.
     */
    public ImageView() {
    	//nothing
    }

    /**
     * This is a callback that will allow us to create the viewer and initialize
     * it.
     */
    @Override
	public void createPartControl(Composite parent) {
        viewer = new ImageViewer(parent);
        makeActions();
        hookContextMenu();
        contributeToActionBars();
    }
    
    private void hookContextMenu() {
        MenuManager menuMgr = new MenuManager("#ImagePopupMenu");
        menuMgr.setRemoveAllWhenShown(true);
        menuMgr.addMenuListener(new IMenuListener() {
            public void menuAboutToShow(IMenuManager manager) {
                ImageView.this.fillContextMenu(manager);
            }
        });
        Menu menu = menuMgr.createContextMenu(viewer.getControl());
        viewer.getControl().setMenu(menu);
        getSite().registerContextMenu(menuMgr, viewer);
    }

    private void contributeToActionBars() {
        IActionBars bars = getViewSite().getActionBars();
        fillLocalPullDown(bars.getMenuManager());
        fillLocalToolBar(bars.getToolBarManager());
    }

    private void fillLocalPullDown(IMenuManager manager) {
    	//nothing
    }

    private void fillContextMenu(IMenuManager manager) {
        manager.add(saveAction);
        manager.add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS));
    }

    private void fillLocalToolBar(IToolBarManager manager) {
        manager.add(saveAction);
    }

    private void makeActions() {
        saveAction = new Action() {
            private IPath queryFile() {
                FileDialog dialog = new FileDialog(viewer.getControl()
                        .getShell(), SWT.SAVE);
                dialog.setFilterExtensions(new String[] { "*.jpg" });
                dialog.setText("Save Image as"); //$NON-NLS-1$
                
                FilesystemUtil.setFileDialogStartPath(dialog);
                String path = dialog.open();
                FilesystemUtil.stickyPathSet(path);
                
                if (path != null && path.length() > 0) {
                    if (!path.endsWith(".jpg"))
                        path = path + ".jpg";
                    return new Path(path);
                }
                return null;
            }

            @Override
			public void run() {
                if (viewer.getImageModel() == null) {
                    MessageDialog.openWarning(viewer.getControl().getShell(),
                            "Error", "No image has been selected");
                    return;
                }
                IPath path = queryFile();
                boolean success = false;
                if (path != null) {
                    success = viewer.getImageModel().saveTo(path);
                    if (!success) {
                        String msg = MessageFormat
                                .format("Cannot write file: {0}",
                                        (Object[])(new String[] { path.toOSString() }));
                        MessageDialog.openWarning(viewer.getControl()
                                .getShell(), "Problem", msg);
                    }
                }
            }
        };
        saveAction.setText("Save Image As");
        saveAction.setToolTipText("Save Image As");
        saveAction.setImageDescriptor(ImageDescriptor.createFromFile(
                org.tekkotsu.ui.storyboard.icons.IconDummy.class, "__save.gif"));
    }

    /**
     * Passing the focus request to the viewer's control.
     */
    @Override
	public void setFocus() {
        viewer.getControl().setFocus();
    }
}